#!/usr/bin/python

from sys     import argv, path
from os.path import join
from time    import time
from random  import seed
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts    import genpsts
from ccnovl  import coconad

#-----------------------------------------------------------------------

def patred (pats):
    pats.sort(key=lambda x: x[2], reverse=True)
    pall = pats                 # sort the patterns and note them
    tmp  = []                   # initialize intermediary pattern list
    while pats:                 # while there are more patterns
        tmp.append(pats[0])     # transfer best pattern to output
        p,c,v = pats[0]         # remove all subset patterns
        pats  = [q for q in pats[1:] if not q[0] <= p]
    for p,c,v in tmp:           # traverse the intermediary patterns
        for q,x,y in pall:      # traverse all patterns
            if y < v: pats.append((p,c)); break
            if q < p: break     # remove all supersets that are worse
    return pats                 # return the reduced pattern list

#-----------------------------------------------------------------------

if __name__ == '__main__':
    ncnt   = 100                # number of neurons/items
    period = 3.0                # recording period
    rate   = 20.0               # (average) firing rate
    size   = 12                 # size of injected assembly
    coin   = 12                 # number of injected coincidences
    jitter = 0.002              # jitter/displacement of coincidences
    width  = 0.003              # influence region/window width
    supp   = 1.0                # minimum support

    seed(1)

    t = time()
    border = dict()
    for i in range(100):
        psts = genpsts(ncnt, period, rate)
        psp  = coconad(list(zip(range(1,ncnt+1), psts)),
                       target='c', width=width, supp=supp, zmin=2,
                       zmax=-1, report='#', algo='i', mode='')
        for z in psp:
            if z not in border or psp[z][1] > border[z]:
                border[z] = psp[z][1]
    t = time() -t
    print('pattern spectrum  : %.2fs' % t)
    print(border)

    t = time()
    asm  = set(range(1,size+1))
    psts = genpsts(ncnt, period, rate, jitter=jitter,
                   asms=[[asm,-coin]])
    pats = coconad(list(zip(range(1,ncnt+1), psts)),
                   target='c', width=width, supp=supp,
                   zmin=2, zmax=-1, report='a', algo='i', mode='')
    t = time() -t
    print('pattern mining    : %.2fs' % t)
    print('number of patterns: %4d pattern(s)' % len(pats))

    t = time()
    upcs = dict()               # and the pattern count dictionaries
    fpcs = dict()               # (unfiltered, filtered, reduced)
    fltd = []; d = e = 0    # generate and mine spike trains
    for p,c in pats:        # and traverse the found patterns
        z = len(p)          # get the size of the pattern
        p = frozenset(p)    # and the set of items/neurons
        if p >= asm:        # check for raw detections
            if z > size: e = 1     # superset detection
            else:        d = 1     # exact    detection
        s = (z,int(c*5))    # compute histogram bin
        if s in upcs: upcs[s] += 1 # count unfiltered patterns
        else:         upcs[s]  = 1 # by their signatures
        if z in border and c <= border[z]: continue
        fltd.append((p,c))  # collect non-excluded patterns
        if s in fpcs: fpcs[s] += 1 # count filtered patterns
        else:         fpcs[s]  = 1 # by their signatures
    pats = [(frozenset(p),c,(len(p)-1)*c) for p,c in pats
            if len(p) not in border or border[len(p)] < c]

    #pats = [(frozenset(p),c,(len(p)-1)*c) for p,c in fltd
    #        if len(p) not in border or border[len(p)] < c]

    t = time() -t
    print('pattern filtering : %.2fs' % t)
    print('after filtering   : %4d pattern(s)' % len(pats))

    t = time()
    for j in range(1000):
        pred = patred(pats)
    t = time() -t
    print('pat.set reduction : %.4fs' % (t/1000))
    print('after reduction   : %4d pattern(s)' % len(pred))

    for p,c in pred: print sorted(p),c
